﻿B4A=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=8
@EndOfDesignText@
Sub Class_Globals
	Type ViaCepErro(IsError As Boolean, ErrorMsg As String, ErrorCode As Long)	
	Type ViaCepTipo(error As ViaCepErro, cep As String, logradouro As String, complemento As String, bairro As String, localidade As String, uf As String, unidade As String, ibge As String, gia As String)	
	
	Private strErro As String = ""	
End Sub

'Initializes the object. You can add parameters to this method if needed.
Public Sub Initialize
	
End Sub

'
'	O webservice 'ViaCep' espera que os campos:
' Sigla, tem 2 caracteres
' Cidade, tem pelo menos 4 caracteres, e,
' Logradouro, tem pelo menos 4 caracteres.
' Se não houver estas condições, o webservice retorna um erro 403.
' Por isto, criei esta função pra validar os campos.
'
Private Sub ValidarEndereco(siglaEstado As String, cidade As String, logradouro As String) As Boolean
	
	If siglaEstado.Trim.Length <> 2 Then
		strErro = "Sigla inválida."
		Return False
	End If
	
	If cidade.Trim.Length <= 3 Then
		strErro = "Deve haver pelo menos 4 caracteres pra o campo cidade."
		Return False
	End If
	
	If logradouro.Trim.Length <= 3 Then
		strErro = "Deve haver pelo menos 4 caracteres pra o campo logradouro."
		Return False
	End If
	
	Return True
	
End Sub

'
' Retorna do webservice, um ou mais endereço com os respectivos cep, da pesquisa
' feita pelo o usuário.
' Nesta função, diferente da função 'ObterEnderecoCep' não iremos retorna um
' tipo 'ViaCepTipo', iremos retorna uma lista de tipos.
'
' Iremos retorna uma lista, se houver erro, a lista retorna um único item
' que tem em um dos campos, a indicação do erro.
'
Public Sub ObterCepDoEndereco(siglaEstado As String, cidade As String, logradouro As String) As ResumableSub
	
	If ValidarEndereco(siglaEstado, cidade, logradouro) = False Then
		Dim objViaCepTipo As ViaCepTipo
		objViaCepTipo.Initialize
	
		objViaCepTipo.error.IsError = True
		objViaCepTipo.error.ErrorMsg = strErro
		
		Dim listaViaCepTipo As List
		listaViaCepTipo.Initialize
		listaViaCepTipo.Add(objViaCepTipo)
		Return listaViaCepTipo
	End If
	
	Dim objHttp As HttpJob
	objHttp.Initialize("", Me)
	objHttp.Download($"https://viacep.com.br/ws/${siglaEstado}/${cidade}/${logradouro}/json/"$)
	Wait For (objHttp) JobDone(objHttp As HttpJob)
	If objHttp.Success = False Then
		objViaCepTipo.error.IsError = True
		If objHttp.ErrorMessage = "<h3>ViaCEP Bad Request (400)</h3>" Then
			objViaCepTipo.error.ErrorMsg = "Formato de cep inválido."
		Else
			objViaCepTipo.error.ErrorMsg = objHttp.ErrorMessage
		End If
		
		objHttp.Release
		Return objViaCepTipo
	End If
	
	Dim listaViaCepTipo As List
	listaViaCepTipo = ConverterJsonPraLista(objHttp.GetString)
	objHttp.Release
	
	Return listaViaCepTipo	
	
End Sub



' Retorna o endereço de um cep, usando o webservice 'ViaCep'.
' Retorna um tipo ViaCep
' @cep Um cep contendo 8 dígitos.
' O webservice espera que o código do cep tenha 8 dígitos
Public Sub ObterEnderecoCep(cep As String) As ResumableSub
	Dim objviacep As ViaCepTipo
	objviacep.Initialize
	
	If Not(ValidarCep(cep)) Then
		objviacep.error.IsError = True
		objviacep.error.ErrorMsg = "Formato de cep inválido, cep deve ter 8 dígitos."
		Return objviacep
	End If
	
	Dim objHttp As HttpJob
	objHttp.Initialize("", Me)
	objHttp.Download($"https://viacep.com.br/ws/${cep}/json"$)
	Wait For (objHttp) JobDone(objHttp As HttpJob)
	If objHttp.Success = False Then
		objviacep.error.IsError = True
		If objHttp.ErrorMessage = "<h3>ViaCEP Bad Request (400)</h3>" Then
			objviacep.error.ErrorMsg = "Formato de cep inválido."
		Else
			objviacep.error.ErrorMsg = objHttp.ErrorMessage
		End If
		
		objHttp.Release
		Return objviacep
	End If
	
	objviacep = ConverterJson(objHttp.GetString)
	objHttp.Release
	Return objviacep
			
End Sub

' 
' O werbservice espera um cep com 8 dígitos, então, devemos, validar
' pra que isto sempre ocorra, antes de enviar a requisição.
'
Private Sub ValidarCep(cep As String) As Boolean
	' No input, pode acontecer do usuário inserir espaços desnecessário,
	' embora o teclado é do tipo numérico, é sempre bom validar.
	cep = cep.Trim()
	
	If Not(IsNumber(cep)) Then Return False
	
	If cep.Length <> 8 Then Return False
	
	Return True
End Sub

'
'
'
Public Sub ConverterJsonPraLista(jsonTexto As String) As List
	Dim listaViaCepTipo As List
	listaViaCepTipo.Initialize	
			
	Dim parser As JSONParser
	parser.Initialize(jsonTexto)
	Dim root As List = parser.NextArray
	For Each colroot As Map In root
		Dim objViaCepTipo As ViaCepTipo
		objViaCepTipo.Initialize	
		
		objViaCepTipo.uf = colroot.Get("uf")
		objViaCepTipo.complemento = colroot.Get("complemento")
		objViaCepTipo.Logradouro = colroot.Get("logradouro")
		objViaCepTipo.bairro = colroot.Get("bairro")
		objViaCepTipo.localidade = colroot.Get("localidade")
		objViaCepTipo.ibge = colroot.Get("ibge")
		objViaCepTipo.unidade = colroot.Get("unidade")
		objViaCepTipo.gia = colroot.get("gia")
		objViaCepTipo.cep = colroot.Get("cep")
		
		listaViaCepTipo.Add(objViaCepTipo)
	Next
	
	Return listaViaCepTipo
End Sub


'
' Converte um string que contém um objeto json em um 
' objeto do tipo ViaCepTipo.
'
Private Sub ConverterJson(jsonTexto As String) As ViaCepTipo
	Dim objviacep As ViaCepTipo
	objviacep.Initialize
	
	' Só iremos chegar aqui, se não houve erro no webservice.
	Dim parser As JSONParser
	parser.Initialize(jsonTexto)
	
	Dim root As Map = parser.NextObject

	' Se o cep é válido, mas não existe o cep, o webservice, vai retorna um objeto,
	' com a chave error.
	If root.ContainsKey("erro") Then
		objviacep.error.IsError = True
		objviacep.error.ErrorMsg = "Erro, Cep não existe."
		Return objviacep
	End If
	
	objviacep.uf = root.Get("uf")
	objviacep.complemento = root.Get("complemento")
	objviacep.Logradouro = root.Get("logradouro")
	objviacep.bairro = root.Get("bairro")
	objviacep.localidade = root.Get("localidade")
	objviacep.ibge = root.Get("ibge")
	objviacep.unidade = root.Get("unidade")
	objviacep.gia = root.get("gia")
	objviacep.cep = root.Get("cep")
	
	Return objviacep
End Sub
